# PIRUS 'PUSH' implementation
# ---------------------------
# 
# Provide data for COUNTER-compliant usage statistics.
#
# Copyright 2012 University of Southampton
#
# Released to the public domain
#
# USE OF THIS EXTENSION IS ENTIRELY AT YOUR OWN RISK
#
# Installation
# ------------
#
# Copy this file into your repository's cfg.d/ directory and restart Apache.
#
# Implementation
# --------------
#
# This code will PING the configured tracker server whenever a full-text object
# is requested from EPrints.
#
# These pings can be aggregated together with data from other sources
# (publishers, other repositories) to create a fuller picture of the usage of
# individual articles.
#
#
# The data transferred are:
#  - eprint.eprintid - the eprint's internal identifier
#  - eprint.datestamp - the datetime the access started
#  - IP address - the user's IP address
#  - User Agent - the user's browser user agent
#  - eprint.date - the publication date
#
# And either (if id_number is defined):
#  - eprint.id_number - the DOI
#
# Or:
#  - eprint.creators_name - first named author
#  - eprint.title - eprint title
#  - eprint.jtitle - publication title
#  - eprint.issn - publication ISSN
#  - eprint.volume - publication volume
#  - eprint.issue - publication issue
#
# Changes
# -------
#
# 1.01
#  - Fixed reference to 'jtitle' instead of 'publication'
#
# 1.00
#  - Initial version

require LWP::UserAgent;
require LWP::ConnCache;

# modify the following URL to the PIRUS tracker location
$c->{pirus}->{tracker} = "http://cclibweb-4.dmz.cranfield.ac.uk/tracker/";

# you may want to revise the settings for the user agent e.g. increase or
# decrease the network timeout
$c->{pirus}->{ua} = LWP::UserAgent->new(
	from => $c->{adminemail},
	agent => $c->{version},
	timeout => 20,
	conn_cache => LWP::ConnCache->new,
);

$c->{plugins}->{"Event::PIRUS"}->{params}->{disable} = 0;

##############################################################################

$c->add_dataset_trigger( 'access', EPrints::Const::EP_TRIGGER_CREATED, sub {
	my( %args ) = @_;

	my $repo = $args{repository};
	my $access = $args{dataobj};

	my $plugin = $repo->plugin( "Event::PIRUS" );

	my $r = $plugin->log( $access );

	if( defined $r && !$r->is_success )
	{
		my $event = $repo->dataset( "event_queue" )->dataobj_class->create_unique( $repo, {
			eventqueueid => Digest::MD5::md5_hex( "Event::PIRUS::replay" ),
			pluginid => "Event::PIRUS",
			action => "replay",
		});
		if( defined $event )
		{
			$event->set_value( "params", [$access->id] );
			$event->commit;
		}
	}
});
